@@ -631,12 +631,24 @@ char *cs_get_exe(void)
 #endif
 	struct file *exe_file;
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
 	if (current->flags & PF_KTHREAD)
 		return kstrdup("<kernel>", GFP_NOFS);
+#else
+	if (segment_eq(get_fs(), KERNEL_DS))
+		return kstrdup("<kernel>", GFP_NOFS);
+#endif
 	mm = current->mm;
 	if (!mm)
 		goto task_has_no_mm;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 1, 0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 15, 0)
+	/* Not using get_mm_exe_file() as it is not exported. */
+	rcu_read_lock();
+	exe_file = rcu_dereference(mm->exe_file);
+	if (exe_file && !get_file_rcu(exe_file))
+		exe_file = NULL;
+	rcu_read_unlock();
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(4, 1, 0)
 	exe_file = get_mm_exe_file(mm);
 #else
 	down_read(&mm->mmap_sem);
